Class: Iro::Position
- Inherits:
-
Object
- Object
- Iro::Position
- Includes:
- Mongoid::Document, Mongoid::Timestamps
- Defined in:
- app/models/iro/position.rb
Constant Summary collapse
- STATUS_ACTIVE =
'active'- STATUS_CLOSED =
'closed'- STATUS_PREPARE =
'prepare'- STATUS_PROPOSED =
'proposed'- STATUS_PENDING =
one more, 'selected' after proposed?
'pending'- STATUSES =
'working'
[ nil, STATUS_CLOSED, STATUS_ACTIVE, STATUS_PREPARE, STATUS_PROPOSED, STATUS_PENDING ]
Class Method Summary collapse
Instance Method Summary collapse
-
#autonxt ⇒ Object
2026-02-26 using this one.
- #begin_delta ⇒ Object
- #breakeven ⇒ Object
- #breakeven_covered_call ⇒ Object
- #breakeven_diag_long_call_spread ⇒ Object
- #breakeven_diag_short_put_spread ⇒ Object
-
#breakeven_long_credit_put_spread ⇒ Object
2026-02-23.
- #breakeven_long_debit_call_spread ⇒ Object
-
#breakeven_short_credit_call_spread ⇒ Object
2026-02-23.
- #breakeven_short_debit_put_spread ⇒ Object
- #calc_nxt ⇒ Object
-
#calc_rollp ⇒ Object
should_roll?.
-
#close_price ⇒ Object
credit spread only.
- #current_underlying_strike ⇒ Object
- #diag_weeks ⇒ Object
- #end_delta ⇒ Object
-
#inner ⇒ Object
Options.
-
#inner_strike ⇒ Object
2026-02-24 only to make finding easier.
-
#inners ⇒ Object
for history and diagonals.
-
#max_gain ⇒ Object
each.
-
#max_loss ⇒ Object
each.
-
#net_amount ⇒ Object
each.
- #net_amount_covered_call ⇒ Object
- #net_amount_diag_long_call_spread ⇒ Object
- #net_amount_diag_short_put_spread ⇒ Object
-
#net_amount_long_credit_put_spread ⇒ Object
2025-10-14 tested.
-
#net_amount_short_credit_call_spread ⇒ Object
2026-02-19 tested.
- #net_amount_short_debit_put_spread ⇒ Object
- #net_percent ⇒ Object
-
#next_expires_on ⇒ Object
ok.
-
#next_reasons ⇒ Object
decisions.
-
#open_price ⇒ Object
credit-spread.
-
#outer_strike ⇒ Object
2026-02-24 only to make finding easier.
-
#outers ⇒ Object
for history and diagonals.
-
#prev ⇒ Object
there are many of these, for viewing on the 'roll' view.
- #put_call ⇒ Object
- #q ⇒ Object
-
#realized_gain_loss_amount ⇒ Object
for diagonals only.
- #realized_gl ⇒ Object
- #refresh ⇒ Object
- #roll_price ⇒ Object
- #schwab_query ⇒ Object
- #status_label(st) ⇒ Object
- #sync ⇒ Object
- #ticker ⇒ Object
- #to_s ⇒ Object
- #trim_expires_on ⇒ Object
Class Method Details
.long ⇒ Object
ok
476 477 478 |
# File 'app/models/iro/position.rb', line 476 def self.long where( long_or_short: Iro::Strategy::LONG ) end |
.short ⇒ Object
ok
481 482 483 |
# File 'app/models/iro/position.rb', line 481 def self.short where( long_or_short: Iro::Strategy::SHORT ) end |
.sync_all ⇒ Object
270 271 272 273 274 275 276 277 278 279 280 281 282 283 284 285 286 287 288 289 290 291 292 293 294 295 296 297 298 299 300 301 302 303 |
# File 'app/models/iro/position.rb', line 270 def self.sync_all @positions = Iro::Position.where( :status.in => [ 'active', 'pending' ] ) expiration_dates = @positions.map { |p| p.expires_on.to_s }.sort # puts! expiration_dates, 'expiration_dates' count = 1 @positions.each do |pos| # puts! pos.id.to_s, '#sync_all.pos' quotes_h = Tda::Option.get_quotes_h({ contractType: 'ALL', ticker: pos.ticker, fromDate: expiration_dates.first, toDate: expiration_dates.last, }) pos.inner.end_price = quotes_h[pos.expires_on.to_date.to_s][pos.put_call][pos.inner.strike][:price] pos.inner.end_delta = quotes_h[pos.expires_on.to_date.to_s][pos.put_call][pos.inner.strike][:delta] pos.inner.save ? print("#{count}^") : print("#{count}X") # if [ Iro::Strategy::KIND_LONG_CREDIT_PUT_SPREAD, # Iro::Strategy::KIND_SHORT_CREDIT_CALL_SPREAD, # Iro::Strategy::KIND_DIAG_LONG_CALL_SPREAD, # Iro::Strategy::KIND_DIAG_SHORT_PUT_SPREAD ].include?( pos.strategy.kind ) if pos.outer pos.outer.end_price = quotes_h[pos.expires_on.to_date.to_s][pos.put_call][pos.outer.strike][:price] pos.outer.end_delta = quotes_h[pos.expires_on.to_date.to_s][pos.put_call][pos.outer.strike][:delta] pos.outer.save ? print('^') : print('X') end count = count+1 end print 'synced-all.' end |
Instance Method Details
#autonxt ⇒ Object
2026-02-26 using this one.
74 |
# File 'app/models/iro/position.rb', line 74 belongs_to :autonxt, class_name: 'Iro::Position', inverse_of: :autoprev, optional: true |
#begin_delta ⇒ Object
119 120 121 122 |
# File 'app/models/iro/position.rb', line 119 def begin_delta # strategy.send("begin_delta_#{strategy.kind}", self) strategy.begin_delta self end |
#breakeven ⇒ Object
128 129 130 |
# File 'app/models/iro/position.rb', line 128 def breakeven send("breakeven_#{strategy.kind}") end |
#breakeven_covered_call ⇒ Object
131 132 133 134 |
# File 'app/models/iro/position.rb', line 131 def breakeven_covered_call p = self p.inner.strike + p.inner.begin_price end |
#breakeven_diag_long_call_spread ⇒ Object
153 154 155 156 |
# File 'app/models/iro/position.rb', line 153 def breakeven_diag_long_call_spread p = self realized_gl + p.outer.strike + p.outer.begin_price - p.inner.begin_price end |
#breakeven_diag_short_put_spread ⇒ Object
157 158 159 160 |
# File 'app/models/iro/position.rb', line 157 def breakeven_diag_short_put_spread p = self p.inner.strike + p.max_gain + p.realized_gl end |
#breakeven_long_credit_put_spread ⇒ Object
2026-02-23
149 150 151 152 |
# File 'app/models/iro/position.rb', line 149 def breakeven_long_credit_put_spread p = self p.inner.strike - p.max_gain end |
#breakeven_long_debit_call_spread ⇒ Object
135 136 137 138 |
# File 'app/models/iro/position.rb', line 135 def breakeven_long_debit_call_spread p = self p.inner.strike - p.max_gain end |
#breakeven_short_credit_call_spread ⇒ Object
2026-02-23
140 141 142 143 |
# File 'app/models/iro/position.rb', line 140 def breakeven_short_credit_call_spread p = self p.inner.strike + p.max_gain end |
#breakeven_short_debit_put_spread ⇒ Object
144 145 146 147 |
# File 'app/models/iro/position.rb', line 144 def breakeven_short_debit_put_spread p = self p.inner.strike - p.inner.begin_price + p.outer.begin_price end |
#calc_nxt ⇒ Object
326 327 328 329 330 331 332 333 334 335 336 337 338 339 340 341 342 343 344 345 346 347 348 349 350 351 352 353 354 355 356 357 358 359 360 361 362 363 364 365 366 367 368 369 370 371 372 373 374 375 376 377 378 379 380 381 382 383 384 385 386 387 388 389 390 391 392 393 394 395 396 397 398 399 400 401 402 403 404 405 406 407 408 409 410 411 412 413 414 415 416 417 418 419 420 421 422 423 424 425 426 427 428 429 430 431 432 433 434 435 436 437 438 439 440 441 442 443 444 445 446 447 448 449 450 451 452 453 454 455 456 457 458 459 460 461 462 |
# File 'app/models/iro/position.rb', line 326 def calc_nxt pos = self # puts! pos, '#calc_nxt...' ## 7 days ahead - not configurable params = { contractType: pos.put_call, expirationDate: next_expires_on, ticker: ticker, } # puts! params, 'ze params' outs = Tda::Option.get_quotes(params) # puts! outs, 'outs' outs_bk = outs.dup ## cleanup mid-increments outs = outs.select do |out| ( out[:strikePrice] - pos.inner.strike ) % strategy.stock. == 0 end outs = outs.select do |out| out[:bidSize] + out[:askSize] > 0 end if 'CALL' == pos.put_call ; elsif 'PUT' == pos.put_call outs = outs.reverse end # puts! outs, '#calc_nxt.outs -> 2' ## next_inner_strike if strategy.next_inner_strike.present? outs = outs.select do |out| if Iro::Strategy::CREDIT == pos.credit_or_debit if Iro::Strategy::SHORT == pos.long_or_short ## short credit call out[:strikePrice] >= strategy.next_inner_strike elsif Iro::Strategy::LONG == pos.long_or_short ## long credit put out[:strikePrice] <= strategy.next_inner_strike end else raise 'zt3 - @TODO: implement, debit spreads' end end # puts! outs[0][:strikePrice], 'after calc next_inner_strike' # puts! outs, 'outs' end ## next_usd_above_mark outs = outs.select do |out| if Iro::Strategy::SHORT == pos.long_or_short out[:strikePrice] > strategy.next_usd_above_mark + strategy.stock.last elsif Iro::Strategy::LONG == pos.long_or_short out[:strikePrice] < strategy.stock.last - strategy.next_usd_above_mark else raise 'zt4 - this cannot happen' end end # puts! outs[0][:strikePrice], 'after calc next_usd_above_mark' # puts! outs, 'outs' ## next_inner_delta outs = outs.select do |out| out_delta = out[:delta].abs rescue 0 out_delta <= strategy.next_inner_delta end # puts! outs[0][:strikePrice], 'after calc next_inner_delta' # puts! outs, 'outs' inner = outs[0] outs = outs.select do |out| if 'CALL' == pos.put_call out[:strikePrice] >= inner[:strikePrice].to_f + strategy.next_spread_amount elsif 'PUT' == pos.put_call out[:strikePrice] <= inner[:strikePrice].to_f - strategy.next_spread_amount end end outer = outs[0] if inner && outer o_attrs = { expires_on: next_expires_on, put_call: pos.put_call, stock_id: pos.stock_id, } inner_attrs = o_attrs.merge({ strike: inner[:strikePrice], begin_price: ( inner[:bid] + inner[:ask] )/2, begin_delta: inner[:delta], end_price: ( inner[:bid] + inner[:ask] )/2, end_delta: inner[:delta], }) outer_attrs = o_attrs.merge({ strike: outer[:strikePrice], begin_price: ( outer[:bid] + outer[:ask] )/2, begin_delta: outer[:delta], end_price: ( outer[:bid] + outer[:ask] )/2, end_delta: outer[:delta], }) autonxt_attrs = { put_call: pos.put_call, status: 'proposed', stock: strategy.stock, inner_strike: inner_attrs[:strike], inner_attributes: inner_attrs, outer_strike: outer_attrs[:strike], outer_attributes: outer_attrs, begin_on: Time.now.to_date, expires_on: next_expires_on, purse: purse, strategy: strategy, quantity: pos.quantity, autoprev: pos, } pos.autonxt ||= Iro::Position.where({ inner_strike: inner_attrs[:strike], outer_strike: outer_attrs[:strike], purse: purse, stock: strategy.stock, strategy: strategy, }).first pos.autonxt ||= Iro::Position.new(autonxt_attrs) pos.autonxt.update(autonxt_attrs) pos.autonxt.inner.update(inner_attrs) pos.autonxt.outer.update(outer_attrs) pos.autonxt.sync pos.autonxt.save! pos.save return pos else throw 'zmq - should not happen' end end |
#calc_rollp ⇒ Object
should_roll?
315 316 317 318 319 320 321 322 323 324 |
# File 'app/models/iro/position.rb', line 315 def calc_rollp pos = self pos.next_reasons = [] out = strategy.send("calc_rollp_#{strategy.kind}", pos ) pos.rollp = out[0] pos.next_reasons.push out[1] save end |
#close_price ⇒ Object
credit spread only
185 186 187 188 189 |
# File 'app/models/iro/position.rb', line 185 def close_price pos = self out = pos.outer.end_price - pos.inner.end_price return out.round(2) end |
#current_underlying_strike ⇒ Object
163 164 165 |
# File 'app/models/iro/position.rb', line 163 def Iro::Stock.find_by( ticker: ticker ).last end |
#diag_weeks ⇒ Object
114 115 116 117 |
# File 'app/models/iro/position.rb', line 114 def diag_weeks pos = self ((pos.outer.expires_on - pos.inner.expires_on)/7).to_i end |
#end_delta ⇒ Object
123 124 125 126 |
# File 'app/models/iro/position.rb', line 123 def end_delta # strategy.send("end_delta_#{strategy.kind}", self) strategy.end_delta self end |
#inner ⇒ Object
Options
80 |
# File 'app/models/iro/position.rb', line 80 belongs_to :inner, class_name: 'Iro::Option', inverse_of: :pos_of_inner |
#inner_strike ⇒ Object
2026-02-24 only to make finding easier.
94 |
# File 'app/models/iro/position.rb', line 94 validates :inner_strike, presence: true |
#inners ⇒ Object
for history and diagonals
82 |
# File 'app/models/iro/position.rb', line 82 has_many :inners, class_name: 'Iro::Option', inverse_of: :poss_of_inner |
#max_gain ⇒ Object
each
232 233 234 |
# File 'app/models/iro/position.rb', line 232 def max_gain # each strategy.send("max_gain_#{strategy.kind}", self) end |
#max_loss ⇒ Object
each
235 236 237 |
# File 'app/models/iro/position.rb', line 235 def max_loss # each strategy.send("max_loss_#{strategy.kind}", self) end |
#net_amount ⇒ Object
each
208 209 210 |
# File 'app/models/iro/position.rb', line 208 def net_amount # each self.send("net_amount_#{strategy.kind}") end |
#net_amount_covered_call ⇒ Object
211 212 213 |
# File 'app/models/iro/position.rb', line 211 def net_amount_covered_call inner.begin_price - inner.end_price end |
#net_amount_diag_long_call_spread ⇒ Object
225 226 227 |
# File 'app/models/iro/position.rb', line 225 def net_amount_diag_long_call_spread inner.begin_price - outer.begin_price + outer.end_price - inner.end_price + realized_gl end |
#net_amount_diag_short_put_spread ⇒ Object
228 229 230 |
# File 'app/models/iro/position.rb', line 228 def net_amount_diag_short_put_spread net_amount_diag_long_call_spread end |
#net_amount_long_credit_put_spread ⇒ Object
2025-10-14 tested
215 216 217 |
# File 'app/models/iro/position.rb', line 215 def net_amount_long_credit_put_spread ## each inner.begin_price - outer.begin_price + outer.end_price - inner.end_price end |
#net_amount_short_credit_call_spread ⇒ Object
2026-02-19 tested
219 220 221 |
# File 'app/models/iro/position.rb', line 219 def net_amount_short_credit_call_spread return net_amount_long_credit_put_spread end |
#net_amount_short_debit_put_spread ⇒ Object
222 223 224 |
# File 'app/models/iro/position.rb', line 222 def net_amount_short_debit_put_spread inner.end_price - inner.begin_price + outer.begin_price - outer.end_price end |
#net_percent ⇒ Object
205 206 207 |
# File 'app/models/iro/position.rb', line 205 def net_percent net_amount / max_gain end |
#next_expires_on ⇒ Object
ok
467 468 469 470 471 472 473 |
# File 'app/models/iro/position.rb', line 467 def next_expires_on out = expires_on.to_datetime.next_occurring(:monday).next_occurring(:friday) if !out.workday? out = Time.previous_business_day(out) end return out.strftime('%Y-%m-%d') end |
#next_reasons ⇒ Object
decisions
311 |
# File 'app/models/iro/position.rb', line 311 field :next_reasons, type: :array, default: [] |
#open_price ⇒ Object
credit-spread
192 193 194 195 196 |
# File 'app/models/iro/position.rb', line 192 def open_price pos = self out = pos.inner.begin_price - pos.outer.begin_price return out.round(2) end |
#outer_strike ⇒ Object
2026-02-24 only to make finding easier.
91 |
# File 'app/models/iro/position.rb', line 91 validates :outer_strike, presence: true |
#outers ⇒ Object
for history and diagonals
86 |
# File 'app/models/iro/position.rb', line 86 has_many :outers, class_name: 'Iro::Option', inverse_of: :poss_of_outer |
#prev ⇒ Object
there are many of these, for viewing on the 'roll' view
70 |
# File 'app/models/iro/position.rb', line 70 belongs_to :prev, class_name: 'Iro::Position', inverse_of: :nxts, optional: true |
#put_call ⇒ Object
61 62 63 |
# File 'app/models/iro/position.rb', line 61 def put_call self[:put_call] || self.strategy.put_call end |
#q ⇒ Object
105 |
# File 'app/models/iro/position.rb', line 105 def q; quantity; end |
#realized_gain_loss_amount ⇒ Object
for diagonals only
8 |
# File 'app/models/iro/position.rb', line 8 field :realized_gain_loss_amount, type: :float, default: 0.0 |
#realized_gl ⇒ Object
9 |
# File 'app/models/iro/position.rb', line 9 def realized_gl; realized_gain_loss_amount; end |
#refresh ⇒ Object
167 168 169 170 171 172 173 174 175 176 177 178 179 |
# File 'app/models/iro/position.rb', line 167 def refresh out = Tda::Option.get_quote({ contractType: 'CALL', strike: strike, expirationDate: expires_on, ticker: ticker, }) update({ end_delta: out[:delta], end_price: out[:last], }) print '^' end |
#roll_price ⇒ Object
198 199 200 201 202 |
# File 'app/models/iro/position.rb', line 198 def roll_price pos = self out = pos.autoprev.outer.end_price - pos.autoprev.inner.end_price + pos.inner.begin_price - pos.outer.begin_price return out.round(2) end |
#schwab_query ⇒ Object
257 258 259 260 261 262 263 264 265 266 267 268 |
# File 'app/models/iro/position.rb', line 257 def schwab_query pos = self case pos.intent when Iro::Strategy::INTENT_OPEN the_q = Tda::Order.credit_spread_q pos when Iro::Strategy::INTENT_ROLL the_q = Tda::Order.roll_credit_call_spread_q pos else throw "prp - #schwab_query undefined for position #{pos.inspect}" end return the_q end |
#status_label(st) ⇒ Object
23 24 25 26 27 |
# File 'app/models/iro/position.rb', line 23 def status_label st labels = {} labels[STATUS_PROPOSED] = 'Selected.' return labels[st] || st end |
#sync ⇒ Object
240 241 242 243 244 245 246 247 248 249 250 251 252 253 254 255 |
# File 'app/models/iro/position.rb', line 240 def sync if schwab_order_id outs = Tda::Order.check_status schwab_order_id update({ schwab_status: outs['status'] }) if [ Tda::Order::STATUS_FILLED, Tda::Order::STATUS_REPLACED ].include?( outs['status'] ) ## update amounts. purse.update({ available_amount: purse.available_amount + next_gain_loss_amount*quantity*100 }) ## make this one active update({ status: Iro::Position::STATUS_ACTIVE, next_gain_loss_amount: nil }) ## make previous one closed autoprev.update({ status: Iro::Position::STATUS_CLOSED }) end end inner.sync outer.sync end |
#ticker ⇒ Object
42 43 44 45 46 47 48 |
# File 'app/models/iro/position.rb', line 42 def ticker if !self[:ticker] self[:ticker] = stock.ticker self.save end self[:ticker] end |
#to_s ⇒ Object
485 486 487 488 489 490 491 492 493 494 495 496 497 498 499 500 501 502 |
# File 'app/models/iro/position.rb', line 485 def to_s out = "#{stock} (#{q}) #{expires_on.to_datetime.strftime('%b %d')} #{strategy.long_or_short} [" if Iro::Strategy::KIND_SHORT_DEBIT_PUT_SPREAD == strategy.kind out = out + "$#{outer.strike} << $#{inner.strike}" elsif Iro::Strategy::LONG == long_or_short if outer&.strike out = out + "$#{outer.strike} << " end out = out + "$#{inner.strike}" else out = out + "$#{inner.strike}" if outer&.strike out = out + " >> $#{outer.strike}" end end out += "] " return out end |
#trim_expires_on ⇒ Object
99 100 101 |
# File 'app/models/iro/position.rb', line 99 def trim_expires_on self.expires_on = expires_on.to_s[0, 10] if expires_on.present? end |